@@ -43,7 +43,7 @@ if TYPE_CHECKING:
 	_: Any
 
 
-__version__ = "2.8.1"
+__version__ = "2.8.2"
 storage['__version__'] = __version__
 
 # add the custom _ as a builtin, it can now be used anywhere in the
@@ -60,10 +60,6 @@ debug(f"Graphics devices detected: {SysInfo._graphics_devices().keys()}")
 # For support reasons, we'll log the disk layout pre installation to match against post-installation layout
 debug(f"Disk states before installing: {disk.disk_layouts()}")
 
-if 'sphinx' not in sys.modules and os.getuid() != 0:
-	print(_("Archinstall requires root privileges to run. See --help for more."))
-	exit(1)
-
 parser = ArgumentParser()
 
 
@@ -96,6 +92,16 @@ def define_arguments():
 						help="Skip the version check when running archinstall")
 
 
+if 'sphinx' not in sys.modules:
+	if '--help' in sys.argv or '-h' in sys.argv:
+		define_arguments()
+		parser.print_help()
+		exit(0)
+	if os.getuid() != 0:
+		print(_("Archinstall requires root privileges to run. See --help for more."))
+		exit(1)
+
+
 def parse_unspecified_argument_list(unknowns: list, multiple: bool = False, err: bool = False) -> dict:
 	"""We accept arguments not defined to the parser. (arguments "ad hoc").
 	Internally argparse return to us a list of words so we have to parse its contents, manually.
@@ -112,12 +118,13 @@ def parse_unspecified_argument_list(unknowns: list, multiple: bool = False, err:
 	argument value value ...
 	which isn't am error if multiple is specified
 	"""
-	tmp_list = unknowns[:]  # wastes a few bytes, but avoids any collateral effect of the destructive nature of the pop method()
+	tmp_list = [arg for arg in unknowns if arg != "="]  # wastes a few bytes, but avoids any collateral effect of the destructive nature of the pop method()
 	config = {}
 	key = None
 	last_key = None
 	while tmp_list:
 		element = tmp_list.pop(0)  # retrieve an element of the list
+
 		if element.startswith('--'):  # is an argument ?
 			if '=' in element:  # uses the arg=value syntax ?
 				key, value = [x.strip() for x in element[2:].split('=', 1)]
@@ -127,23 +134,19 @@ def parse_unspecified_argument_list(unknowns: list, multiple: bool = False, err:
 			else:
 				key = element[2:]
 				config[key] = True  # every argument starts its lifecycle as boolean
-		else:
-			if element == '=':
-				continue
-			if key:
-				config[key] = element
-				last_key = key  # multiple
-				key = None
+		elif key:
+			config[key] = element
+			last_key = key  # multiple
+			key = None
+		elif multiple and last_key:
+			if isinstance(config[last_key], str):
+				config[last_key] = [config[last_key], element]
 			else:
-				if multiple and last_key:
-					if isinstance(config[last_key], str):
-						config[last_key] = [config[last_key], element]
-					else:
-						config[last_key].append(element)
-				elif err:
-					raise ValueError(f"Entry {element} is not related to any argument")
-				else:
-					print(f" We ignore the entry {element} as it isn't related to any argument")
+				config[last_key].append(element)
+		elif err:
+			raise ValueError(f"Entry {element} is not related to any argument")
+		else:
+			print(f" We ignore the entry {element} as it isn't related to any argument")
 	return config
 
 
